@@ -1,19 +1,20 @@
-use std::error::{Error as StdError};
+use std::error::Error as StdError;
 use std::fmt;
 use std::iter;
 use std::num;
 use std::str;
 
-use serde::de::{
-    Deserializer, DeserializeSeed, Deserialize, IntoDeserializer,
-    Error as SerdeError, Unexpected,
-    Visitor, EnumAccess, VariantAccess, MapAccess, SeqAccess,
-};
 use serde::de::value::BorrowedBytesDeserializer;
+use serde::de::{
+    Deserialize, DeserializeSeed, Deserializer, EnumAccess,
+    Error as SerdeError, IntoDeserializer, MapAccess, SeqAccess, Unexpected,
+    VariantAccess, Visitor,
+};
+use serde::serde_if_integer128;
 
-use byte_record::{ByteRecord, ByteRecordIter};
-use error::{Error, ErrorKind, new_error};
-use string_record::{StringRecord, StringRecordIter};
+use crate::byte_record::{ByteRecord, ByteRecordIter};
+use crate::error::{Error, ErrorKind};
+use crate::string_record::{StringRecord, StringRecordIter};
 
 use self::DeserializeErrorKind as DEK;
 
@@ -27,7 +28,7 @@ pub fn deserialize_string_record<'de, D: Deserialize<'de>>(
         field: 0,
     });
     D::deserialize(&mut deser).map_err(|err| {
-        new_error(ErrorKind::Deserialize {
+        Error::new(ErrorKind::Deserialize {
             pos: record.position().map(Clone::clone),
             err: err,
         })
@@ -44,7 +45,7 @@ pub fn deserialize_byte_record<'de, D: Deserialize<'de>>(
         field: 0,
     });
     D::deserialize(&mut deser).map_err(|err| {
-        new_error(ErrorKind::Deserialize {
+        Error::new(ErrorKind::Deserialize {
             pos: record.position().map(Clone::clone),
             err: err,
         })
@@ -181,7 +182,7 @@ impl<'r> DeRecord<'r> for DeStringRecord<'r> {
             None => Err(DeserializeError {
                 field: None,
                 kind: DEK::UnexpectedEndOfRow,
-            })
+            }),
         }
     }
 
@@ -208,14 +209,22 @@ impl<'r> DeRecord<'r> for DeStringRecord<'r> {
     ) -> Result<V::Value, DeserializeError> {
         let x = self.next_field()?;
         if x == "true" {
-            visitor.visit_bool(true)
+            return visitor.visit_bool(true);
         } else if x == "false" {
-            visitor.visit_bool(false)
-        } else if let Some(n) = try_positive_integer(x) {
-            visitor.visit_u64(n)
-        } else if let Some(n) = try_negative_integer(x) {
-            visitor.visit_i64(n)
-        } else if let Some(n) = try_float(x) {
+            return visitor.visit_bool(false);
+        } else if let Some(n) = try_positive_integer64(x) {
+            return visitor.visit_u64(n);
+        } else if let Some(n) = try_negative_integer64(x) {
+            return visitor.visit_i64(n);
+        }
+        serde_if_integer128! {
+            if let Some(n) = try_positive_integer128(x) {
+                return visitor.visit_u128(n);
+            } else if let Some(n) = try_negative_integer128(x) {
+                return visitor.visit_i128(n);
+            }
+        }
+        if let Some(n) = try_float(x) {
             visitor.visit_f64(n)
         } else {
             visitor.visit_str(x)
@@ -238,9 +247,10 @@ impl<'r> DeRecord<'r> for DeByteRecord<'r> {
     #[inline]
     fn next_header(&mut self) -> Result<Option<&'r str>, DeserializeError> {
         match self.next_header_bytes() {
-            Ok(Some(field)) => Ok(Some(str::from_utf8(field).map_err(|err| {
-                self.error(DEK::InvalidUtf8(err))
-            })?)),
+            Ok(Some(field)) => Ok(Some(
+                str::from_utf8(field)
+                    .map_err(|err| self.error(DEK::InvalidUtf8(err)))?,
+            )),
             Ok(None) => Ok(None),
             Err(err) => Err(err),
         }
@@ -256,9 +266,8 @@ impl<'r> DeRecord<'r> for DeByteRecord<'r> {
     #[inline]
     fn next_field(&mut self) -> Result<&'r str, DeserializeError> {
         self.next_field_bytes().and_then(|field| {
-            str::from_utf8(field).map_err(|err| {
-                self.error(DEK::InvalidUtf8(err))
-            })
+            str::from_utf8(field)
+                .map_err(|err| self.error(DEK::InvalidUtf8(err)))
         })
     }
 
@@ -272,7 +281,7 @@ impl<'r> DeRecord<'r> for DeByteRecord<'r> {
             None => Err(DeserializeError {
                 field: None,
                 kind: DEK::UnexpectedEndOfRow,
-            })
+            }),
         }
     }
 
@@ -294,14 +303,22 @@ impl<'r> DeRecord<'r> for DeByteRecord<'r> {
     ) -> Result<V::Value, DeserializeError> {
         let x = self.next_field_bytes()?;
         if x == b"true" {
-            visitor.visit_bool(true)
+            return visitor.visit_bool(true);
         } else if x == b"false" {
-            visitor.visit_bool(false)
-        } else if let Some(n) = try_positive_integer_bytes(x) {
-            visitor.visit_u64(n)
-        } else if let Some(n) = try_negative_integer_bytes(x) {
-            visitor.visit_i64(n)
-        } else if let Some(n) = try_float_bytes(x) {
+            return visitor.visit_bool(false);
+        } else if let Some(n) = try_positive_integer64_bytes(x) {
+            return visitor.visit_u64(n);
+        } else if let Some(n) = try_negative_integer64_bytes(x) {
+            return visitor.visit_i64(n);
+        }
+        serde_if_integer128! {
+            if let Some(n) = try_positive_integer128_bytes(x) {
+                return visitor.visit_u128(n);
+            } else if let Some(n) = try_negative_integer128_bytes(x) {
+                return visitor.visit_i128(n);
+            }
+        }
+        if let Some(n) = try_float_bytes(x) {
             visitor.visit_f64(n)
         } else if let Ok(s) = str::from_utf8(x) {
             visitor.visit_str(s)
@@ -312,14 +329,19 @@ impl<'r> DeRecord<'r> for DeByteRecord<'r> {
 }
 
 macro_rules! deserialize_int {
-    ($method:ident, $visit:ident) => {
+    ($method:ident, $visit:ident, $inttype:ty) => {
         fn $method<V: Visitor<'de>>(
             self,
             visitor: V,
         ) -> Result<V::Value, Self::Error> {
-            visitor.$visit(
-                self.next_field()?
-                    .parse().map_err(|err| self.error(DEK::ParseInt(err)))?)
+            let field = self.next_field()?;
+            let num =
+                if field.starts_with("0x") {
+                    <$inttype>::from_str_radix(&field[2..], 16)
+                } else {
+                    field.parse()
+                };
+            visitor.$visit(num.map_err(|err| self.error(DEK::ParseInt(err)))?)
         }
     }
 }
@@ -342,17 +364,25 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
     ) -> Result<V::Value, Self::Error> {
         visitor.visit_bool(
             self.next_field()?
-                .parse().map_err(|err| self.error(DEK::ParseBool(err)))?)
+                .parse()
+                .map_err(|err| self.error(DEK::ParseBool(err)))?,
+        )
     }
 
-    deserialize_int!(deserialize_u8, visit_u8);
-    deserialize_int!(deserialize_u16, visit_u16);
-    deserialize_int!(deserialize_u32, visit_u32);
-    deserialize_int!(deserialize_u64, visit_u64);
-    deserialize_int!(deserialize_i8, visit_i8);
-    deserialize_int!(deserialize_i16, visit_i16);
-    deserialize_int!(deserialize_i32, visit_i32);
-    deserialize_int!(deserialize_i64, visit_i64);
+    deserialize_int!(deserialize_u8, visit_u8, u8);
+    deserialize_int!(deserialize_u16, visit_u16, u16);
+    deserialize_int!(deserialize_u32, visit_u32, u32);
+    deserialize_int!(deserialize_u64, visit_u64, u64);
+    serde_if_integer128! {
+        deserialize_int!(deserialize_u128, visit_u128, u128);
+    }
+    deserialize_int!(deserialize_i8, visit_i8, i8);
+    deserialize_int!(deserialize_i16, visit_i16, i16);
+    deserialize_int!(deserialize_i32, visit_i32, i32);
+    deserialize_int!(deserialize_i64, visit_i64, i64);
+    serde_if_integer128! {
+        deserialize_int!(deserialize_i128, visit_i128, i128);
+    }
 
     fn deserialize_f32<V: Visitor<'de>>(
         self,
@@ -360,7 +390,9 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
     ) -> Result<V::Value, Self::Error> {
         visitor.visit_f32(
             self.next_field()?
-                .parse().map_err(|err| self.error(DEK::ParseFloat(err)))?)
+                .parse()
+                .map_err(|err| self.error(DEK::ParseFloat(err)))?,
+        )
     }
 
     fn deserialize_f64<V: Visitor<'de>>(
@@ -369,7 +401,9 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
     ) -> Result<V::Value, Self::Error> {
         visitor.visit_f64(
             self.next_field()?
-                .parse().map_err(|err| self.error(DEK::ParseFloat(err)))?)
+                .parse()
+                .map_err(|err| self.error(DEK::ParseFloat(err)))?,
+        )
     }
 
     fn deserialize_char<V: Visitor<'de>>(
@@ -381,7 +415,8 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
         if len != 1 {
             return Err(self.error(DEK::Message(format!(
                 "expected single character but got {} characters in '{}'",
-                len, field))));
+                len, field
+            ))));
         }
         visitor.visit_char(field.chars().next().unwrap())
     }
@@ -404,9 +439,7 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
         self,
         visitor: V,
     ) -> Result<V::Value, Self::Error> {
-        self.next_field_bytes().and_then(|f| {
-            visitor.visit_borrowed_bytes(f)
-        })
+        self.next_field_bytes().and_then(|f| visitor.visit_borrowed_bytes(f))
     }
 
     fn deserialize_byte_buf<V: Visitor<'de>>(
@@ -545,8 +578,8 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> EnumAccess<'de>
     }
 }
 
-impl<'a, 'de: 'a, T: DeRecord<'de>>
-    VariantAccess<'de> for &'a mut DeRecordWrap<T>
+impl<'a, 'de: 'a, T: DeRecord<'de>> VariantAccess<'de>
+    for &'a mut DeRecordWrap<T>
 {
     type Error = DeserializeError;
 
@@ -581,8 +614,8 @@ impl<'a, 'de: 'a, T: DeRecord<'de>>
     }
 }
 
-impl<'a, 'de: 'a, T: DeRecord<'de>>
-    SeqAccess<'de> for &'a mut DeRecordWrap<T>
+impl<'a, 'de: 'a, T: DeRecord<'de>> SeqAccess<'de>
+    for &'a mut DeRecordWrap<T>
 {
     type Error = DeserializeError;
 
@@ -598,8 +631,8 @@ impl<'a, 'de: 'a, T: DeRecord<'de>>
     }
 }
 
-impl<'a, 'de: 'a, T: DeRecord<'de>>
-    MapAccess<'de> for &'a mut DeRecordWrap<T>
+impl<'a, 'de: 'a, T: DeRecord<'de>> MapAccess<'de>
+    for &'a mut DeRecordWrap<T>
 {
     type Error = DeserializeError;
 
@@ -654,10 +687,7 @@ pub enum DeserializeErrorKind {
 
 impl SerdeError for DeserializeError {
     fn custom<T: fmt::Display>(msg: T) -> DeserializeError {
-        DeserializeError {
-            field: None,
-            kind: DEK::Message(msg.to_string()),
-        }
+        DeserializeError { field: None, kind: DEK::Message(msg.to_string()) }
     }
 }
 
@@ -723,11 +753,21 @@ impl DeserializeErrorKind {
     }
 }
 
-fn try_positive_integer(s: &str) -> Option<u64> {
+serde_if_integer128! {
+    fn try_positive_integer128(s: &str) -> Option<u128> {
+        s.parse().ok()
+    }
+
+    fn try_negative_integer128(s: &str) -> Option<i128> {
+        s.parse().ok()
+    }
+}
+
+fn try_positive_integer64(s: &str) -> Option<u64> {
     s.parse().ok()
 }
 
-fn try_negative_integer(s: &str) -> Option<i64> {
+fn try_negative_integer64(s: &str) -> Option<i64> {
     s.parse().ok()
 }
 
@@ -735,14 +775,24 @@ fn try_float(s: &str) -> Option<f64> {
     s.parse().ok()
 }
 
-fn try_positive_integer_bytes(s: &[u8]) -> Option<u64> {
+fn try_positive_integer64_bytes(s: &[u8]) -> Option<u64> {
     str::from_utf8(s).ok().and_then(|s| s.parse().ok())
 }
 
-fn try_negative_integer_bytes(s: &[u8]) -> Option<i64> {
+fn try_negative_integer64_bytes(s: &[u8]) -> Option<i64> {
     str::from_utf8(s).ok().and_then(|s| s.parse().ok())
 }
 
+serde_if_integer128! {
+    fn try_positive_integer128_bytes(s: &[u8]) -> Option<u128> {
+        str::from_utf8(s).ok().and_then(|s| s.parse().ok())
+    }
+
+    fn try_negative_integer128_bytes(s: &[u8]) -> Option<i128> {
+        str::from_utf8(s).ok().and_then(|s| s.parse().ok())
+    }
+}
+
 fn try_float_bytes(s: &[u8]) -> Option<f64> {
     str::from_utf8(s).ok().and_then(|s| s.parse().ok())
 }
@@ -751,17 +801,15 @@ fn try_float_bytes(s: &[u8]) -> Option<f64> {
 mod tests {
     use std::collections::HashMap;
 
-    use serde::de::DeserializeOwned;
-    use serde_bytes::{self, ByteBuf};
+    use bstr::BString;
+    use serde::{de::DeserializeOwned, serde_if_integer128, Deserialize};
 
-    use byte_record::ByteRecord;
-    use error::Error;
-    use string_record::StringRecord;
     use super::{deserialize_byte_record, deserialize_string_record};
+    use crate::byte_record::ByteRecord;
+    use crate::error::Error;
+    use crate::string_record::StringRecord;
 
-    fn de<D: DeserializeOwned>(
-        fields: &[&str],
-    ) -> Result<D, Error> {
+    fn de<D: DeserializeOwned>(fields: &[&str]) -> Result<D, Error> {
         let record = StringRecord::from(fields);
         deserialize_string_record(&record, None)
     }
@@ -788,10 +836,8 @@ mod tests {
             x: String,
         }
 
-        let got: Foo = de_headers(
-            &["x", "y", "z"],
-            &["hi", "42", "1.3"],
-        ).unwrap();
+        let got: Foo =
+            de_headers(&["x", "y", "z"], &["hi", "42", "1.3"]).unwrap();
         assert_eq!(got, Foo { x: "hi".into(), y: 42, z: 1.3 });
     }
 
@@ -807,7 +853,8 @@ mod tests {
         assert!(de_headers::<Foo>(
             &["a", "x", "y", "z"],
             &["foo", "hi", "42", "1.3"],
-        ).is_err());
+        )
+        .is_err());
     }
 
     #[test]
@@ -818,10 +865,7 @@ mod tests {
             y: i32,
             x: String,
         }
-        assert!(de_headers::<Foo>(
-            &["y", "z"],
-            &["42", "1.3"],
-        ).is_err());
+        assert!(de_headers::<Foo>(&["y", "z"], &["42", "1.3"],).is_err());
     }
 
     #[test]
@@ -833,10 +877,7 @@ mod tests {
             x: Option<String>,
         }
 
-        let got: Foo = de_headers(
-            &["y", "z"],
-            &["42", "1.3"],
-        ).unwrap();
+        let got: Foo = de_headers(&["y", "z"], &["42", "1.3"]).unwrap();
         assert_eq!(got, Foo { x: None, y: 42, z: 1.3 });
     }
 
@@ -878,7 +919,7 @@ mod tests {
         assert_eq!(got.unwrap(), Foo);
 
         let got = de_headers::<Bar>(&[], &[]);
-        assert_eq!(got.unwrap(), Bar{});
+        assert_eq!(got.unwrap(), Bar {});
 
         let got = de_headers::<()>(&[], &[]);
         assert_eq!(got.unwrap(), ());
@@ -908,6 +949,14 @@ mod tests {
         assert_eq!(got, 42);
     }
 
+    serde_if_integer128! {
+        #[test]
+        fn one_field_128() {
+            let got: i128 = de(&["2010223372036854775808"]).unwrap();
+            assert_eq!(got, 2010223372036854775808);
+        }
+    }
+
     #[test]
     fn two_fields() {
         let got: (i32, bool) = de(&["42", "true"]).unwrap();
@@ -953,6 +1002,23 @@ mod tests {
         assert_eq!(got, vec![1, 5, 10]);
     }
 
+    #[test]
+    fn simple_hex_seq() {
+        let got: Vec<i32> = de(&["0x7F", "0xA9", "0x10"]).unwrap();
+        assert_eq!(got, vec![0x7F, 0xA9, 0x10]);
+    }
+
+    #[test]
+    fn mixed_hex_seq() {
+        let got: Vec<i32> = de(&["0x7F", "0xA9", "10"]).unwrap();
+        assert_eq!(got, vec![0x7F, 0xA9, 10]);
+    }
+
+    #[test]
+    fn bad_hex_seq() {
+        assert!(de::<Vec<u8>>(&["7F", "0xA9", "10"]).is_err());
+    }
+
     #[test]
     fn seq_in_struct() {
         #[derive(Deserialize, Debug, PartialEq)]
@@ -992,7 +1058,7 @@ mod tests {
 
     #[test]
     fn bytes() {
-        let got: Vec<u8> = de::<ByteBuf>(&["foobar"]).unwrap().into();
+        let got: Vec<u8> = de::<BString>(&["foobar"]).unwrap().into();
         assert_eq!(got, b"foobar".to_vec());
     }
 
@@ -1040,15 +1106,16 @@ mod tests {
             String(String),
         }
 
-        let got: Row = de_headers(
-            &["x", "y", "z"],
-            &["true", "null", "1"],
-        ).unwrap();
-        assert_eq!(got, Row {
-            x: Boolish::Bool(true),
-            y: Boolish::String("null".into()),
-            z: Boolish::Number(1),
-        });
+        let got: Row =
+            de_headers(&["x", "y", "z"], &["true", "null", "1"]).unwrap();
+        assert_eq!(
+            got,
+            Row {
+                x: Boolish::Bool(true),
+                y: Boolish::String("null".into()),
+                z: Boolish::Number(1),
+            }
+        );
     }
 
     #[test]
@@ -1060,10 +1127,8 @@ mod tests {
             c: Option<i32>,
         }
 
-        let got: Foo = de_headers(
-            &["a", "b", "c"],
-            &["", "foo", "5"],
-        ).unwrap();
+        let got: Foo =
+            de_headers(&["a", "b", "c"], &["", "foo", "5"]).unwrap();
         assert_eq!(got, Foo { a: None, b: "foo".into(), c: Some(5) });
     }
 
@@ -1071,18 +1136,16 @@ mod tests {
     fn option_invalid_field() {
         #[derive(Deserialize, Debug, PartialEq)]
         struct Foo {
-            #[serde(deserialize_with = "::invalid_option")]
+            #[serde(deserialize_with = "crate::invalid_option")]
             a: Option<i32>,
-            #[serde(deserialize_with = "::invalid_option")]
+            #[serde(deserialize_with = "crate::invalid_option")]
             b: Option<i32>,
-            #[serde(deserialize_with = "::invalid_option")]
+            #[serde(deserialize_with = "crate::invalid_option")]
             c: Option<i32>,
         }
 
-        let got: Foo = de_headers(
-            &["a", "b", "c"],
-            &["xyz", "", "5"],
-        ).unwrap();
+        let got: Foo =
+            de_headers(&["a", "b", "c"], &["xyz", "", "5"]).unwrap();
         assert_eq!(got, Foo { a: None, b: None, c: Some(5) });
     }
 
@@ -1097,8 +1160,8 @@ mod tests {
 
         let headers = StringRecord::from(vec!["a", "b", "c"]);
         let record = StringRecord::from(vec!["foo", "5", "bar"]);
-        let got: Foo = deserialize_string_record(
-            &record, Some(&headers)).unwrap();
+        let got: Foo =
+            deserialize_string_record(&record, Some(&headers)).unwrap();
         assert_eq!(got, Foo { a: "foo", b: 5, c: "bar" });
     }
 
@@ -1108,13 +1171,11 @@ mod tests {
 
         let headers = StringRecord::from(vec!["a", "b", "c"]);
         let record = StringRecord::from(vec!["aardvark", "bee", "cat"]);
-        let got: HashMap<&str, &str> = deserialize_string_record(
-            &record, Some(&headers)).unwrap();
+        let got: HashMap<&str, &str> =
+            deserialize_string_record(&record, Some(&headers)).unwrap();
 
-        let expected: HashMap<&str, &str> = headers
-            .iter()
-            .zip(&record)
-            .collect();
+        let expected: HashMap<&str, &str> =
+            headers.iter().zip(&record).collect();
         assert_eq!(got, expected);
     }
 
@@ -1124,13 +1185,11 @@ mod tests {
 
         let headers = ByteRecord::from(vec![b"a", b"\xFF", b"c"]);
         let record = ByteRecord::from(vec!["aardvark", "bee", "cat"]);
-        let got: HashMap<&[u8], &[u8]> = deserialize_byte_record(
-            &record, Some(&headers)).unwrap();
+        let got: HashMap<&[u8], &[u8]> =
+            deserialize_byte_record(&record, Some(&headers)).unwrap();
 
-        let expected: HashMap<&[u8], &[u8]> = headers
-            .iter()
-            .zip(&record)
-            .collect();
+        let expected: HashMap<&[u8], &[u8]> =
+            headers.iter().zip(&record).collect();
         assert_eq!(got, expected);
     }
 
@@ -1159,10 +1218,13 @@ mod tests {
         let header = StringRecord::from(vec!["x", "y", "prop1", "prop2"]);
         let record = StringRecord::from(vec!["1", "2", "3", "4"]);
         let got: Row = record.deserialize(Some(&header)).unwrap();
-        assert_eq!(got, Row {
-            input: Input { x: 1.0, y: 2.0 },
-            properties: Properties { prop1: 3.0, prop2: 4.0 },
-        });
+        assert_eq!(
+            got,
+            Row {
+                input: Input { x: 1.0, y: 2.0 },
+                properties: Properties { prop1: 3.0, prop2: 4.0 },
+            }
+        );
     }
 
     #[test]
@@ -1170,22 +1232,22 @@ mod tests {
         #[derive(Debug, Deserialize, PartialEq)]
         struct Row {
             h1: String,
-            #[serde(with = "serde_bytes")]
-            h2: Vec<u8>,
+            h2: BString,
             h3: String,
         }
 
         let headers = ByteRecord::from(vec![b"h1", b"h2", b"h3"]);
-        let record = ByteRecord::from(vec![
-            b(b"baz"), b(b"foo\xFFbar"), b(b"quux"),
-        ]);
-        let got: Row = deserialize_byte_record(
-            &record, Some(&headers),
-        ).unwrap();
-        assert_eq!(got, Row {
-            h1: "baz".to_string(),
-            h2: b"foo\xFFbar".to_vec(),
-            h3: "quux".to_string(),
-        });
+        let record =
+            ByteRecord::from(vec![b(b"baz"), b(b"foo\xFFbar"), b(b"quux")]);
+        let got: Row =
+            deserialize_byte_record(&record, Some(&headers)).unwrap();
+        assert_eq!(
+            got,
+            Row {
+                h1: "baz".to_string(),
+                h2: BString::from(b"foo\xFFbar".to_vec()),
+                h3: "quux".to_string(),
+            }
+        );
     }
 }
